package com.plannaplan.responses.mappers;

import static org.junit.Assert.assertTrue;

import java.util.Arrays;
import java.util.List;

import com.plannaplan.entities.Groups;
import com.plannaplan.entities.Lecturer;
import com.plannaplan.responses.models.DefaultGroupResponse;
import com.plannaplan.responses.models.GetCourseGroupsResponse;
import com.plannaplan.responses.models.WithCapacityGroupResponse;
import com.plannaplan.types.WeekDay;

import org.junit.Test;

public class GroupsMappersTest {
    @Test
    public void shouldMapToResponseIncludingCapacity() {
        final List<Groups> gropus = Arrays.asList(
                new Groups(42, "A4-1", null, 520, WeekDay.MONDAY, new Lecturer("krul.", "Wladyslaw", "Potocki")));
        final List<WithCapacityGroupResponse> response = GroupsMappers.mapToCapacityResponse(gropus);

        assert (response.get(0).getCapacity() == 42);
        assert (response.get(0) instanceof WithCapacityGroupResponse);
        assert (response.size() == 1);
    }

    @Test
    public void shouldMapToResponseWiothoutCapacity() {
        final List<Groups> gropus = Arrays.asList(
                new Groups(42, "A4-1", null, 520, WeekDay.MONDAY, new Lecturer("krul.", "Wladyslaw", "Potocki")));
        final List<DefaultGroupResponse> response = GroupsMappers.mapToDefaultResponse(gropus);

        assert (response.get(0) instanceof DefaultGroupResponse);
        assert (response.size() == 1);
    }

    @Test
    public void shouldMapToGetCourseGroupsResponse(){
        final List<Groups> groups = Arrays.asList(
            new Groups(150, "A4-1", null, 520, WeekDay.MONDAY, new Lecturer("krul.", "Wladyslaw", "Potocki")), new Groups(24, "A4-1", null, 520, WeekDay.MONDAY, new Lecturer("krul.", "Wladyslaw", "Potocki")));
        
        final GetCourseGroupsResponse<DefaultGroupResponse> response = GroupsMappers.mapToGetCourseGroupsDefaultResponse(groups);

        assertTrue(response.getClasses().size() == 1);
        assertTrue(response.getLectures().size() == 1);

        }
        
}

